<?php
/**
 * dtweb - Device Tree editor for UDOO boards
 * Copyright (C) 2015 Francesco Montefoschi <francesco.monte@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package dtweb
 * @author  Francesco Montefoschi
 * @license http://www.gnu.org/licenses/gpl-3.0.html  GNU GPL 3.0
 */

class Model_DeviceTree
{
    public $features;
    public $gpios;
    public $dtsi;
    public $requiredLines;
    public $regulators = "";
    public $regulatorPinctrls = "";
    public $regulatorPinctrlsSNVS = "";
    
    public function disableEverything()
    {
        foreach ($this->features as $feature) {
            $feature->disabled = true;
        }
        
        foreach ($this->gpios->gpios as $n => $gpio) {
            $this->gpios->enable($n);
        }
    }
    
    public function setFeatures(array $features)
    {
        $this->features = $features;
    }
    
    public function setGpios(Model_Gpios $gpios)
    {
        $this->gpios = $gpios;
    }
    
    public function setDtsi(array $dtsi)
    {
        $this->dtsi = $dtsi;
    }
    
    public function setRequiredLines(array $requiredLines)
    {
        $this->requiredLines = $requiredLines;
    }
    
    public function getFeature($name)
    {
        foreach ($this->features as $feature) {
            if ($feature->name == $name) {
                return $feature;
            }
        }
    }
    
    public function enableFeature($name)
    {
        $feature = $this->getFeature($name);
        $feature->disabled = false;

        if (array_key_exists($name, $this->requiredLines)) {
            foreach ($this->requiredLines[$name] as $lineToUncomment) {
                if (preg_match('/^\/\//', $this->dtsi[$lineToUncomment])) {
                    $row = substr($this->dtsi[$lineToUncomment], 2);
                    $this->dtsi[$lineToUncomment] = $row;
                }
            }
        }
    }

    private function extractIndent($line) {
        $trimmed = ltrim($line);
	return str_replace($trimmed, '', $line);
    }

    public function setProperties($name, $props) {
        if (array_key_exists($name, $this->requiredLines)) {
            foreach ($props as $key => $val) {
                $searchStr = $name . "/" . $key;
                $preg = "/" . preg_quote($searchStr, "/") . ".*?}}/";
                foreach ($this->requiredLines[$name] as $lineToUncomment) {
                    if (preg_match($preg, $this->dtsi[$lineToUncomment], $matches)) {
                        if ($val === "") {
                            $this->dtsi[$lineToUncomment] = "// " . $this->dtsi[$lineToUncomment];
                            continue;
                        }
                        $indent = $this->extractIndent($this->dtsi[$lineToUncomment]);
                        $exploded = explode("/", str_replace(array("val", "}}"), array($val, ";"), $matches[0]));
                        $row = end($exploded);
                        $this->dtsi[$lineToUncomment] = $indent . $row;
                    }
                }
            }
        }
    }

    public function enableFunctionPins($name, array $pins)
    {
        foreach ($this->dtsi as &$line) {
            foreach ($pins as $pin) {
                if (strpos($line, '{{external-'.$name.'-'.$pin.'}}')) {
                    $row = substr($line, 2);
		    $line = $row;
                }
            }
        }
    }
    
    public function disableGpios(array $pins)
    {
        foreach ($pins as $pin) {
            $this->gpios->disable($pin);
        }
    }

    public function enableRegulators($name, array $pins, array $props, array $snvspins)
    {
        if ($name != "reg") {
            return;
        }
        $origDtsDir = realpath(__DIR__ . '/../../dts');
        $pinctrlBase = file_get_contents($origDtsDir . "/regulator_pinctrl_base");
        $regBase = file_get_contents($origDtsDir . "/regulator_base");

        foreach ($pins as $pin) {
            $pinVal = $this->gpios->gpios[$pin]['value'];
            preg_match("/\s*(\/\/)?\s*(MX6[A-Z0-9_]*).+GPIO(\d)_IO(\d+)/", $pinVal, $matches);
            $pinName = $matches[0];
            $search = ["{{no}}", "{{pin}}"];
            $replace = [$pin, $pinName];
            if (in_array($pin, $snvspins)) {
                $this->regulatorPinctrlsSNVS .= str_replace($search, $replace, $pinctrlBase);
            } else {
                $this->regulatorPinctrls .= str_replace($search, $replace, $pinctrlBase);
            }

            preg_match("/GPIO(\d)_IO(\d+)/", $pinName, $matches);
            $gno = substr($matches[0], 4, 1);
            $iono = intval(substr($matches[0], 8));
            $search = ["{{no}}", "{{gno}}", "{{iono}}"];
            $replace = [$pin, $gno, $iono];
            $regBase = str_replace($search, $replace, $regBase);
            if (isset($props[$name . "_" . $pin])) {
                foreach ($props[$name . "_" . $pin] as $propName => $val) {
                    if ($val === "FALSE") {
                        $regBase = preg_replace("/^.*" . $propName .";$/m", "//", $regBase);
                    } else {
                        $regBase = preg_replace("/" . $propName .".*?<(\d+)>;/", $propName . " = <" . $val . ">;", $regBase);
                    }
                }
            }
            $this->regulators .= $regBase;
        }
    }
}
